@@ -1,16 +1,65 @@
+import json
 import requests
 import os
+from tabulate import tabulate
 
 
 class NexusDB:
     def __init__(self, api_key=None):
         self.base_url = os.environ.get("BASE_URL", "https://api.nexusdb.io/query")
-        # Use the provided API key or fallback to the environment variable
         self.api_key = (
             api_key if api_key is not None else os.environ.get("NEXUSDB_API_KEY")
         )
+
         self.headers = {"Content-Type": "application/json", "API-Key": self.api_key}
 
+    def _process_response(self, response, tabulate_option=False):
+        if not response.text:
+            print("Error: Empty response from server")
+            return
+
+        try:
+            response_data = response.json()
+            if (
+                tabulate_option
+                and "headers" in response_data
+                and "rows" in response_data
+            ):
+                # Simplify and prepare rows for tabulation
+                simplified_rows = []
+                for row in response_data["rows"]:
+                    simplified_row = []
+                    for cell in row:
+                        # Simplify data structure for Num and Str types, or fallback to the original structure
+                        if isinstance(cell, dict):
+                            if "Num" in cell and "Int" in cell["Num"]:
+                                simplified_row.append(cell["Num"]["Int"])
+                            elif "Str" in cell:
+                                simplified_row.append(cell["Str"])
+                            else:
+                                simplified_value = next(
+                                    iter(cell.values()), str(cell)
+                                )  # Attempt to simplify further or convert to string
+                                if isinstance(simplified_value, dict):
+                                    simplified_row.append(
+                                        next(
+                                            iter(simplified_value.values()),
+                                            str(simplified_value),
+                                        )
+                                    )
+                                else:
+                                    simplified_row.append(simplified_value)
+                        else:
+                            simplified_row.append(cell)
+                    simplified_rows.append(simplified_row)
+
+                return tabulate(simplified_rows, headers=response_data["headers"])
+            else:
+                return response.text
+        except json.JSONDecodeError:
+            print(f"Error: Response: {response.text} could not be decoded as JSON")
+            return response.text
+
     def create(self, relation_name, columns):
         """Creates a new relation with the specified columns, making adjustments for optional parameters."""
         formatted_columns = []
@@ -90,9 +139,7 @@ class NexusDB:
 
         return response.text
 
-    def lookup(self, relation_name, fields=None, condition=""):
-        """Looks up data from the specified relation."""
-        # Using fields=None and then setting it to [] if None to avoid mutable default argument
+    def lookup(self, relation_name, fields=None, condition="", tabulate=False):
         if fields is None:
             fields = []
 
@@ -103,9 +150,9 @@ class NexusDB:
             "condition": condition,
         }
         response = requests.post(self.base_url, headers=self.headers, json=data)
-        return response.text
+        return self._process_response(response, tabulate)
 
-    def join(self, join_type, relations, return_fields, option=None):
+    def join(self, join_type, relations, return_fields, option=None, tabulate=False):
         """
         Executes a join query with the specified parameters.
 
@@ -129,17 +176,15 @@ class NexusDB:
             data["return"]["option"] = option
 
         response = requests.post(self.base_url, headers=self.headers, json=data)
-        return response.text
+        return self._process_response(response, tabulate)
 
-    def delete(self, relation_name, primary_keys):
-        """Deletes data from the specified relation based on primary keys."""
-        # Convert primary_keys dict to the expected list format
-        primary_keys_list = [{"name": k, "value": v} for k, v in primary_keys.items()]
+    def delete(self, relation_name, condition):
+        """Deletes data from the specified relation where condition is met."""
 
         data = {
             "query_type": "Delete",
             "relation_name": relation_name,
-            "primary_keys": primary_keys_list,
+            "condition": condition,
         }
         print(f"data: {data}\n\n")
         response = requests.post(self.base_url, headers=self.headers, json=data)
@@ -178,3 +223,59 @@ class NexusDB:
 
         # Return the server's response
         return response.text
+
+    def insert_with_vector(
+        self,
+        relation_name,
+        text,
+        vectors,
+        access_keys=None,
+        metadata=None,
+        references=None,
+    ):
+        payload = {
+            "query_type": "Insert",
+            "relation_name": relation_name,
+            "searchable_content": {
+                "text": text,
+                "vectors": vectors,
+            },
+        }
+        # Only add optional parameters to the payload if they are not None
+        if access_keys is not None:
+            payload["searchable_content"]["access_keys"] = access_keys
+        if metadata is not None:
+            payload["searchable_content"]["metadata"] = metadata
+        if references is not None:
+            payload["searchable_content"]["reference"] = references
+
+        response = requests.post(self.base_url, json=payload, headers=self.headers)
+        return response.text
+
+    def vector_search(
+        self,
+        query_vector,
+        access_keys=None,
+        search_radius=None,
+        number_of_results=None,
+        filter_statement=None,
+        tabulate=False,
+    ):
+        query_payload = {
+            "query_type": "VectorSearch",
+            "query_vector": query_vector,
+        }
+
+        if access_keys is not None:
+            query_payload["access_keys"] = access_keys
+        if search_radius is not None:
+            query_payload["search_radius"] = search_radius
+        if number_of_results is not None:
+            query_payload["number_of_results"] = number_of_results
+        if filter_statement is not None:
+            query_payload["filter_statement"] = filter_statement
+
+        response = requests.post(
+            self.base_url, json=query_payload, headers=self.headers
+        )
+        return self._process_response(response, tabulate)
